const uswds = require("@uswds/compile");

// Custom variables
const pkg = require('./package.json');
const {series, watch, src} = require("gulp");
const gulp = require('gulp');
const styleLint = require('@ronilaukkarinen/gulp-stylelint');
const eslint = require('gulp-eslint');
const rename = require("gulp-rename");
const uglify = require('gulp-uglify-es').default;

/**
 * USWDS version
 */
uswds.settings.version = 3;

/**
 * Path settings
 * Set as many as you need
 */
// Where to output uswds assets.
uswds.paths.dist.css = './assets/css';
uswds.paths.dist.fonts = './assets/uswds/fonts';
uswds.paths.dist.img = './assets/uswds/images';
uswds.paths.dist.js = './assets/uswds/js';
uswds.paths.src.projectSass = './assets/uswds/sass';

// // Custom scss files
uswds.paths.dist.theme = './sass';

/**
 * Fully delete the assets folder.
 *
 * @returns {Promise<string>}
 */
async function cleanAssets() {
  await del([pkg.paths.dist.assets], {
    onProgress: progress => {
      console.log(progress + ' delete');
    }
  });
  return Promise.resolve('assets folder deleted.');
}

/**
 * Delete js folder.
 *
 * @returns {Promise<string>}
 */
function cleanJs() {
  require('del').deleteSync((([pkg.paths.dist.js])));
  return Promise.resolve('js folder deleted.');
}

/**
 * Delete CSS folder.
 *
 * @returns {string}
 */
function cleanCss() {
  deleteSync([
    pkg.paths.dist.css
  ]);
  return 'Deleted';
}

/**
 * Lints the scss files.
 *
 * @returns {*}
 */
function cssLint() {
  return src(pkg.paths.scss)
    .pipe(styleLint({
      failAfterError: false,
      reporters: [
        {formatter: 'string', console: true}
      ]
    }))
}

function cssLintFix() {
  return src(pkg.paths.scss)
    .pipe(styleLint({
      fix: true,
      failAfterError: false,
      reporters: [
        {formatter: 'string', console: true}
      ]
    }))
    .pipe(gulp.dest('./components'));
}

function buildJs() {
  return src(pkg.paths.js)
    .pipe(rename(function (file) {
      // this removes the last parent directory of the relative file path
      let parts = file.dirname.split("/");
      parts = parts.slice(1);
      parts = parts.join("/");
      file.dirname = path.dirname(parts + "/" + file.basename);
    }))
    .pipe(rename({"suffix": '.min'}))
    .pipe(uglify())
    .pipe(gulp.dest(pkg.paths.dist.js));
}

function jsLint(fix = false) {
  return src(pkg.paths.js)
    .pipe(eslint({fix: fix}))
    .pipe(eslint.format())
}

/**
 * Copy all images out of components and place in assets.
 */
function copyImages() {
  return src('./components/**/*.{png,jpg,svg}')
    .pipe(gulp.dest('./assets/images'));
}


// USWDS build exports.
exports.init = uswds.init;
exports.uswdsInit = exports.init;
exports.updateUswds = uswds.updateUswds;
exports.copyUswds = series(uswds.copyAssets);

// Custom exports.
exports.compileCss = series(cssLint, uswds.compileSass);
exports.fixCss = series(cssLintFix, uswds.compileSass);

// Working above
exports.compileJs = series(jsLint, buildJs);
exports.compileAll = series(copyImages, this.compileJs, this.compileCss);
exports.lintJs = series(jsLint);
exports.lintFull = series(cssLint, jsLint);

exports.watch = series(this.copyUswds, this.compileAll, () => {
  watch(pkg.paths.scss, series([this.compileSass]));
  watch(pkg.paths.js, series([this.compileJs]));
});

exports.default = series(this.copyUswds, this.compileAll);
